from .split_list import split_list


class Communicator:
    """
    Wrapper around the MPI4pi that adds some useful methods.
    """
    comm = None
    root = None
    is_root = None
    rank = None
    comm_size = None

    def __init__(self, root=0):
        from mpi4py import MPI
        self.MPI = MPI
        self.root = root
        self.use_communicator(MPI.COMM_WORLD)

        # Processor name
        self.processor_name = MPI.Get_processor_name()

    def use_communicator(self, communicator):
        self.comm = communicator
        # Define rank and communicator size
        self.rank = self.comm.Get_rank()
        self.comm_size = self.comm.Get_size()
        # Determine if processis root
        self.is_root = self.rank == self.root

    def scatter(self, input_list, root=0, round_robin=False):
        # Wrapper to distribute a list even if it does not have the same number of elements
        # than the communicator
        if self.rank == root:
            data = split_list(input_list, self.comm_size, round_robin=round_robin)
        else:
            data = None
        return self.comm.scatter(data, root=root)

    def gather(self, input_lists, root=0):
        # Wrapper to gather lists and merge them in the root
        to_return = self.comm.gather(input_lists, root=root)
        if self.rank == root:
            merged = []
            for l in to_return:
                try:
                    merged.extend(l)
                except TypeError:
                    merged.append(l)
            to_return = merged
        return to_return

    def bcast(self, obj, root=0):
        # Wrapper to broadcast an object from 0
        return self.comm.bcast(obj, root)

    def sync(self):
        self.comm.Barrier()

    def spawn(self, n_workers, scriptname=None):
        import sys
        if scriptname is None:
            # to get main script filename
            import __main__ as main
            filename = main.__file__
        else:
            filename = scriptname

        parent_comm = self.MPI.Comm.Get_parent()
        try:
            parent_comm.Get_size()
            seed = False
        except self.MPI.Exception:
            seed = True

        # The proper way to do it is by using
        # to launch the same copy of the main script

        # However with the jupyter notebook it does not work
        # And we have to launch an script that includes this same
        # instructions.
        # filename = "spawner.py"

        if seed:
            if self.comm_size == 1:
                new_comm = self.comm.Spawn(
                    sys.executable,
                    args=[filename],
                    maxprocs=n_workers)
                merged = new_comm.Merge()
                self.use_communicator(merged)
        else:
            MPI = self.MPI
            new_comm = MPI.Comm.Get_parent()
            merged = new_comm.Merge()
            self.use_communicator(merged)

        self.comm.Barrier()




